require 'browser'

class Transition

  attr_reader :destination

  def self.to(destination, button=nil, fields={})
    new(destination, button, fields)
  end

  def initialize(destination, button, fields)
    @destination = destination
    @button = button
    @fields = fields
  end

  def implement_through(browser)
    @fields.each_pair { | field, value |
      browser.enter(field, value)
    }
    browser.press(@button)
  end

  def has_field?(name)
    @fields.has_key?(name)
  end

  def set_field(name, value)
    @fields[name] = value
  end
    
end

class Path

  Arcs = {}
  Nodes = []

  def self.def_arc_between(from_page, to_page, button, fields={})
    Arcs[from_page] ||= {} 
    from_paths = Arcs[from_page]
    from_paths[to_page] = [button, fields]
    Nodes << from_page
    Nodes << to_page     # yes, lots of duplication. I don't care.
  end

  def_arc_between(:beginning, :main, 'Login',
                  { :login => 'unimportant', :password => 'unimportant' })

  def_arc_between(:main, :add_case_page, "New Case")

  def_arc_between(:add_case_page, :case_display_page, "Record Case",
                  { :client => 'unimportant', :clinic_id => '213'})

  def_arc_between(:case_display_page, :add_visit_page, "Add Visit")

  def_arc_between(:add_visit_page, :case_display_page, "Record Visit",
                  { :diagnosis => 'unimportant',
                    :charges => "100"})

  def_arc_between(:case_display_page, :add_audit_page, "Add Audit")

  def_arc_between(:add_audit_page, :case_display_page, "Record Audit",
                  { :auditor => "unimportant", :variance => 'none' })

  def self.between(from_page, to_page)
    new(from_page, to_page)
  end

  attr_reader :path

  def initialize(from_page, to_page)
    check_pages(from_page, to_page)
    @from_page = from_page
    @to_page = to_page
    @path = find_path
  end

  def ending_page
    @path.last.destination
  end
    

  def check_pages(*pages)
    pages.each { | page |
      unless Nodes.include?(page)
        raise StandardError.new("I don't know about page #{page.inspect}.")
      end
    }
  end

  def find_path()
    return [] if @from_page == @to_page
    extend([[@from_page]])
  end

  def extend(partial_paths)
    extended_paths = []
    partial_paths.each { | partial_path |
      page_reached_so_far = partial_path.last
      unless page_reached_so_far.is_a?(Symbol)
        raise StandardError.new("Corrupt path #{@path}")
      end
      Arcs[page_reached_so_far].each_pair { | destination, mechanism |
        extended_paths << (partial_path +
                           [Transition.to(destination, *mechanism), destination])
        return extended_paths.last if destination == @to_page
      }
    }
    extend(extended_paths)
  end

  def execute
    browser = Browser.new
    browser.go("doesn't matter where")
    @path.each { | node |
      if node.respond_to?(:implement_through)
        node.implement_through(browser)
      end
    }
  end
end
